﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Reflection.Emit;
using System.Text;
using System.Windows.Forms;

namespace TwoDimensionalArrayToGrid
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();

            BindGrid();
        }

        private void BindGrid()
        {
            const int columns = 5;
            const int rows = 5;
            string[,] array = GenerateArray(columns, rows);

            Type t = GenerateDynamicType(columns);

            List<object> dummyList = new List<object>();

            for (int j = 0; j < rows; j++)
            {
                object o1 = Activator.CreateInstance(t);
                PropertyInfo[] pinfos = o1.GetType().GetProperties();

                for (int i = 0; i < columns; i++)
                {
                    pinfos[i].SetValue(o1, array[i,j], null);
                }
                dummyList.Add(o1);
            }

            dataGridView1.DataSource = dummyList;
        }

        private static string[,] GenerateArray(int columns, int rows)
        {
            string[,] array = new string[columns,rows];

            for (int i = 0; i < columns ; i++)
            {
                for (int j = 0; j < rows; j++)
                {
                    array[i, j] = "(" + i + "," + j + ")";
                }    
            }

            return array;
        }

        private static Type GenerateDynamicType(int columnsCount)
        {
            AssemblyName aName = new AssemblyName("DynamicAssemblyExample");
            AssemblyBuilder ab =
                AppDomain.CurrentDomain.DefineDynamicAssembly(
                    aName,
                    AssemblyBuilderAccess.Run);

            // Create the module.
            ModuleBuilder mb = ab.DefineDynamicModule(aName.Name);

            TypeBuilder tb = mb.DefineType(
                "IGDynamicType",
                TypeAttributes.Public);


            for (int i = 0; i < columnsCount; i++)
            {
                string propertyName = "Column" + i;
                Type propertyType = typeof(object);

                // Add a private field
                FieldBuilder fbNumber = tb.DefineField(
                    "m_" + propertyName,
                    propertyType,
                    FieldAttributes.Private);


                // Define a property that gets and sets the private  field.
                //
                // The last argument of DefineProperty is null, because the
                // property has no parameters. (If you don't specify null, you must
                // specify an array of Type objects. For a parameterless property,
                // use the built-in array with no elements: Type.EmptyTypes)
                PropertyBuilder pbNumber = tb.DefineProperty(
                    propertyName,
                    PropertyAttributes.HasDefault,
                    propertyType,
                    null);

                // The property "set" and property "get" methods require a special
                // set of attributes.
                const MethodAttributes getSetAttr = MethodAttributes.Public |
                                                    MethodAttributes.SpecialName | MethodAttributes.HideBySig;

                // Define the "get" accessor method for the property. The method returns
                // the same as the property type and has no arguments. Note that null could be 
                // used instead of Types.EmptyTypes)
                MethodBuilder mbNumberGetAccessor = tb.DefineMethod(
                    "get_" + propertyName,
                    getSetAttr,
                    propertyType,
                    Type.EmptyTypes);

                ILGenerator numberGetIL = mbNumberGetAccessor.GetILGenerator();
                // For an instance property, argument zero is the instance. Load the 
                // instance, then load the private field and return, leaving the
                // field value on the stack.
                numberGetIL.Emit(OpCodes.Ldarg_0);
                numberGetIL.Emit(OpCodes.Ldfld, fbNumber);
                numberGetIL.Emit(OpCodes.Ret);

                // Define the "set" accessor method for the property, which has no return
                // type and takes one argument of type the same as the property type.
                MethodBuilder mbNumberSetAccessor = tb.DefineMethod(
                    "set_" + propertyName,
                    getSetAttr,
                    null,
                    new[] { propertyType });

                ILGenerator numberSetIL = mbNumberSetAccessor.GetILGenerator();
                // Load the instance and then the numeric argument, then store the
                // argument in the field.
                numberSetIL.Emit(OpCodes.Ldarg_0);
                numberSetIL.Emit(OpCodes.Ldarg_1);
                numberSetIL.Emit(OpCodes.Stfld, fbNumber);
                numberSetIL.Emit(OpCodes.Ret);

                // Last, map the "get" and "set" accessor methods to the 
                // PropertyBuilder. The property is now complete. 
                pbNumber.SetGetMethod(mbNumberGetAccessor);
                pbNumber.SetSetMethod(mbNumberSetAccessor);
            }

            // Define a method that accepts an integer argument and returns
            // Finish the type.
            return tb.CreateType();
        }
    }
}
